import java.net.*;
import java.io.*;
import java.util.Vector;
import java.util.ResourceBundle;

public class QueryDHL
{
  private  URL url;
  private  URLConnection uc;

  public void connect ( String urlAddress ) throws IOException, MalformedURLException {
     url = new URL ( urlAddress );
     uc = url.openConnection();
  }//connect

  public TrackingHolder perform ( TrackingHolder holder ) throws Exception {
     try
     {
       //Default is no Delivery Information Available
       holder.setErrorMessage (TrackingDefaults.COMPLETE_MESSAGE [2] );

       ResourceBundle rb = ResourceBundle.getBundle("file");
       StringBuffer sb = new StringBuffer ();
       sb.append(rb.getString("DHL_TRACKING_URI"));
       sb.append("?KEYSTRING=");
       sb.append (holder.getTrackNumber() );
       sb.append("&QUERYTYPE="+rb.getString("QUERY_TYPE"));
       sb.append("&LAN="+rb.getString("LANGUAGE"));
       sb.append("&TID="+rb.getString("TID"));
       sb.append("&FIRST_DB="+rb.getString("FIRST_DB"));
       sb.append("&COMPANYNM="+rb.getString("COMPANYNAME"));
       connect ( sb.toString () );
       holder = read ( holder );
     }
     catch ( Exception e )
     {
        holder.setErrorMessage (TrackingDefaults.COMPLETE_MESSAGE [1] );
        e.printStackTrace ();
     }
     return holder ;
  } // perform

  private TrackingHolder read ( TrackingHolder holder) throws IOException
  {
     String wkString = "";
     try
     {
       StringBuffer    sb = new StringBuffer ( );
       int             size = 0;

       BufferedInputStream  bis = new BufferedInputStream ( uc.getInputStream ( ) );
       byte [] readFileBytes = new byte [ 4096 ];

       while ( true )
       {
         size = bis.read ( readFileBytes, 0, 4096 );

         if ( size != -1)
         {
           sb.append ( new String ( readFileBytes, 0, size ) );
         }
         else break;
       }

       bis.close ();

       wkString = sb.toString ();
     }
     catch ( Exception e )
     {
        e.printStackTrace ();
     }

     DHLTrackParser dtp = new DHLTrackParser();
     dtp.parseURI(wkString);

     DHLSummaryRecord summRec = dtp.getSummaryRecord();
     Vector serviceEvents = dtp.getServiceEvents();
     holder.setStatus (summRec.getLastEventComment ()+mapNoDataInfo(summRec.getNoDataInfo ()));
     holder.setDeliveryLocation (summRec.getDestiCity ()+" "+summRec.getDestiCountry ());
     if(!summRec.getLastEventMonth ().trim().equals (""))
     holder.setDeliveryDate (summRec.getLastEventMonth ()+"/"+summRec.getLastEventDay ()+"/"+
                       summRec.getLastEventYear ()+" "+summRec.getLastEventHour ()+":"+
                       summRec.getLastEventMinute ());
     String eventActivity = "";
     for (int i=0; i<serviceEvents.size (); i++) {
         if( i >0 ) eventActivity += "<br>";
         eventActivity += ((DHLServiceEvent)serviceEvents.elementAt (i)).getEvent();
     }
     holder.setScanActivity (eventActivity);
     holder.setSignedForBy (summRec.getSignatory ());

     return holder;

  } // read

  private String mapNoDataInfo(String str) {
   if(str.trim().equalsIgnoreCase ("No Information Available"))
      return "Shipment information not available";
   else if( str.trim().equalsIgnoreCase("Invalid Request"))
      return "Invalid DHL Tracking number";
   else if(str.trim().equalsIgnoreCase("Service Unavailable"))
      return "DHL Tracking service is not available at this time";
   else
     return str;
  }

  public static void main(String[] args)
  {
    try
    {
      QueryDHL queryDHL = new QueryDHL();
      TrackingHolder holder = new TrackingHolder ();
 
      System.out.println("Please enter a DHL Airway bill");
      BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
      String awb = in.readLine();
      in.close();
      System.out.println("Retrieving Tracking details.");
      if (awb == null || awb.trim().equalsIgnoreCase(""))
         awb = "8959057995";
      holder.setTrackNumber(awb);
      holder = queryDHL.perform ( holder );
      System.out.println (holder.toString() );
    }
    catch (Exception e )
    {
      e.printStackTrace ();
    }
  }
} // QueryUPS
